@@ -300,7 +300,9 @@ def ssh_impl(**kwargs: Any) -> None:
         print(e, file=sys.stderr)
         sys.exit(1)
 
-    instances = scan.find_transient_instances(name=config.name, with_ssh=True)
+    instances = scan.find_transient_instances(
+        name=config.name, timeout=config.ssh_timeout
+    )
     if len(instances) > 1:
         # There shouldn't be instances with the same name, so just take the first and log
         logging.warning(
@@ -313,6 +315,13 @@ def ssh_impl(**kwargs: Any) -> None:
         print(f"No running VMs found with the name '{config.name}'", file=sys.stderr)
         sys.exit(1)
 
+    if instance.ssh_port is None:
+        print(
+            f"Running VM '{config.name}' has no known SSH port. Was it started with '-ssh'?",
+            file=sys.stderr,
+        )
+        sys.exit(1)
+
     ssh_config = ssh.SshConfig(
         host="127.0.0.1",
         user=config.ssh_user,
@@ -321,7 +330,7 @@ def ssh_impl(**kwargs: Any) -> None:
     )
     client = ssh.SshClient(config=ssh_config, command=config.ssh_command)
     connection = client.connect_stdout(config.ssh_timeout)
-    connection.wait()
+    sys.exit(connection.wait())
 
 
 @cli_entry.group("list")
@@ -343,12 +352,15 @@ def list_vm_impl(**kwargs: Any) -> None:
         print(e, file=sys.stderr)
         sys.exit(1)
 
-    instances = scan.find_transient_instances(name=config.name, with_ssh=config.with_ssh)
+    instances = scan.find_transient_instances(
+        name=config.name, with_ssh=config.with_ssh, timeout=None
+    )
     if len(instances) == 0:
         print("No running VMs found matching criteria", file=sys.stderr)
         sys.exit(1)
 
     print(scan.format_instance_table(instances))
+    sys.exit(0)
 
 
 @click.help_option("-h", "--help")
