@@ -59,9 +59,9 @@ class TransientVm:
     config: configuration.Config
     vm_images: Sequence[image.BaseImageInfo]
     ssh_config: Optional[ssh.SshConfig]
-    set_ssh_port: Optional[int]
     qemu_runner: Optional[qemu.QemuRunner]
     qemu_should_die: bool
+    set_ssh_port: Optional[int]
 
     def __init__(self, config: configuration.Config, store: image.ImageStore) -> None:
         self.store = store
@@ -72,6 +72,7 @@ class TransientVm:
         self.qemu_should_die = False
         self.name = self.config.name or self.__generate_tmp_name()
         self.state = TransientVmState.WAITING
+        self.data_tempfile = tempfile.TemporaryFile("wb+", buffering=0)
         self.set_ssh_port = None
 
     def __generate_tmp_name(self) -> str:
@@ -228,17 +229,10 @@ class TransientVm:
                 new_args.extend(["-serial", "stdio", "-display", "none"])
 
             if self.config.ssh_port is None:
-                ssh_port = utils.allocate_random_port()
+                # If the user didn't specify one, let the kernel pick
+                ssh_port = 0
             else:
                 ssh_port = self.config.ssh_port
-            self.set_ssh_port = ssh_port
-
-            self.ssh_config = ssh.SshConfig(
-                host="127.0.0.1",
-                port=self.set_ssh_port,
-                user=self.config.ssh_user,
-                ssh_bin_name=self.config.ssh_bin_name,
-            )
 
             ssh_net_driver = self.config.ssh_net_driver
 
@@ -254,10 +248,27 @@ class TransientVm:
 
         return new_args
 
+    def __prepare_ssh(self) -> None:
+        # Wait until the QMP connection is established (this should be very fast).
+        # We must do this _before_ using ssh, as we use QMP to get the ssh port
+        assert self.qemu_runner is not None
+        assert self.qemu_runner.qmp_client is not None
+        self.qemu_runner.qmp_client.connect(self.config.qmp_timeout)
+
+        if self.config.ssh_port:
+            self.set_ssh_port = self.config.ssh_port
+        else:
+            self.set_ssh_port = ssh.find_ssh_port_forward(self.qemu_runner.qmp_client)
+
+        self.ssh_config = ssh.SshConfig(
+            host="127.0.0.1",
+            port=self.set_ssh_port,
+            user=self.config.ssh_user,
+            ssh_bin_name=self.config.ssh_bin_name,
+        )
+
     def __connect_ssh(self) -> int:
         assert self.ssh_config is not None
-        assert self.qemu_runner is not None
-
         client = ssh.SshClient(config=self.ssh_config, command=self.config.ssh_command)
         conn = client.connect_stdout(timeout=self.config.ssh_timeout)
 
@@ -330,22 +341,22 @@ class TransientVm:
             raise utils.TransientProcessError(returncode=returncode)
         return None
 
-    def __build_qemu_environment(self) -> Dict[str, str]:
-        env = {
-            # Always add the sentinel value
-            scan.SCAN_ENVIRON_SENTINEL: "1",
-            # Some versions of python don't propagate PATH, so always set it
-            "PATH": os.getenv("PATH") or "",
+    def __prepare_proc_data(self) -> None:
+        data = {
+            "name": self.name,
         }
-        data = {"name": self.name}
 
-        if self.set_ssh_port is not None:
+        if self.__needs_ssh():
             data["ssh_port"] = self.set_ssh_port
 
         data_json_bytes = json.dumps(data).encode("utf-8")
-        data_encoded = base64.b64encode(data_json_bytes).decode("utf-8")
-        env[scan.SCAN_ENVIRON_DATA] = data_encoded
-        return env
+        self.data_tempfile.write(base64.b64encode(data_json_bytes))
+
+    def __build_qemu_environment(self) -> Dict[str, str]:
+        qemu_env = os.environ.copy()
+        qemu_env[scan.SCAN_ENVIRON_SENTINEL] = "1"
+        qemu_env[scan.SCAN_DATA_FD] = str(self.data_tempfile.fileno())
+        return qemu_env
 
     def run(self) -> None:
         self.state = TransientVmState.RUNNING
@@ -377,16 +388,17 @@ class TransientVm:
             qemu_interactive = False
             qemu_quiet = not self.config.ssh_with_serial
 
-        # Note that we must _not_ use QMP if we aren't using the SSH connection, because
-        # passing the `-qmp` arg causes QEMU to terminate on SIGINT, even when in
-        # `-nographic` mode, which is very surprising.
+        # Note that we must _not_ use QMP if we aren't using the SSH connection,
+        # because passing the `-qmp` arg causes QEMU to terminate on SIGINT, even
+        # when in `-nographic` mode, which is very surprising.
         self.qemu_runner = qemu.QemuRunner(
             full_qemu_args,
             bin_name=self.config.qemu_bin_name,
             quiet=qemu_quiet,
             interactive=qemu_interactive,
-            qmp_connectable=self.__needs_ssh_console(),
+            qmp_connectable=self.__needs_ssh(),
             env=self.__build_qemu_environment(),
+            pass_fds=(self.data_tempfile.fileno(),),
         )
 
         qemu_proc = self.qemu_runner.start()
@@ -399,6 +411,13 @@ class TransientVm:
             logging.error("QEMU Process has died. Exiting")
             return self.__post_run(qemu_returncode)
 
+        if self.__needs_ssh():
+            self.__prepare_ssh()
+
+        # Now that we know the ssh port (if any), we can write out the
+        # proc data to our tempfile
+        self.__prepare_proc_data()
+
         sshfs_threads: List[checked_threading.Thread] = []
         for shared_spec in self.config.shared_folder:
             assert self.ssh_config is not None
@@ -428,10 +447,6 @@ class TransientVm:
             sshfs_thread.join()
 
         if self.__needs_ssh_console():
-            # Now wait until the QMP connection is established (this should be very fast).
-            assert self.qemu_runner.qmp_client is not None
-            self.qemu_runner.qmp_client.connect(self.config.qmp_timeout)
-
             # Note that we always return the SSH exit code, even if the guest failed to
             # shut down. This ensures the shutdown_timeout=0 case is handled as expected.
             # (i.e., it returns the SSH code instead of a QEMU error)
